/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.pp.fimi;

import java.io.*;
import java.util.*;
import java.util.Map.*;

import rst.util.data.PairIntInt;

/** Manages coding of a table column. */
public class ColumnCode 
{
	/** Column name. */
	public String id;
	/** Column domain. */
	private TreeMap<String, PairIntInt> values; // PairIntInt is (code, number of occurrences)
	/** Starting code number. */
	private int base;
	public int sequence = 1;
	
	/** Constructor. */
	public ColumnCode(String id) 
	{
		this.id = id;
		values = new TreeMap<String, PairIntInt>();
		base = 0;
	}
	/** Set starting code. */
	public void setBase(int base) 
	{
		this.base = base;
	}
	/** Add a code for a value. */
	public void addCode(String val)
	{
		PairIntInt map = values.get(val);
		if( map != null ) {
			map.second++;
			return;
		}
		map = new PairIntInt(sequence++, 1);
		values.put(val, map );
	}
	/** Return code of a value. */
	public int getCode(String val) throws RuntimeException
	{
		PairIntInt map = values.get(val);
		if( map != null )
			return base + map.first;
		throw new RuntimeException("Decode item not found: " + val);
	}
	/** Output codings. */
	public void output(PrintWriter out)
	{
		Iterator<Entry<String, PairIntInt>> i = values.entrySet().iterator();
		while (i.hasNext()) {
			Entry<String, PairIntInt> e = i.next();
			String key = e.getKey();
			if(key.equals(""))
				key = "NULL";
			int value = base + e.getValue().first;
			int support = e.getValue().second;
			out.println(value + "\t" + id + "="  + key + "\t" + support);
		} 
	}
	/** Remove items with support greater than or equal to maxs. */
	public void removeHighSupport(int maxs) {
		Iterator<Entry<String, PairIntInt>> i = values.entrySet().iterator();
		while (i.hasNext()) {
			Entry<String, PairIntInt> e = i.next();
			if( e.getValue().second >= maxs )
				i.remove();
		} 
	}
}
