/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.table;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;

import rst.util.data.Bitmap;
import rst.util.IOUtil;

/** Table as bitmaps of attributes. */
public class BitTable {
	/** Attributes. */
	public Bitmap []atts;
	
	/** Constructor. */
	public BitTable(int ncols, int nrows) {
		atts = new Bitmap[ncols];
		for(int i=0; i< ncols; ++i)
			atts[i] = new Bitmap(nrows);
	}
	/** Load table from FIMI file. */
	public void loadFromFimi(String fileName) throws IOException {
		loadFromFimi(fileName, " ");
	}
	/** Load table from FIMI file. */
	public void loadFromFimi(String fileName, String sep) throws IOException {
		for(int i=0; i< atts.length; ++i) 
			atts[i].unsetall();
		BufferedReader in = IOUtil.getReader(fileName);
		String []line;
		int pos = 0;
		while( (line=IOUtil.readTokens(in, sep)) != null ) {
			for(int i=0; i<line.length; ++i) {
				int item = Integer.parseInt(line[i]);
				atts[item].set(pos);
			}
			++pos;
		}
		in.close();
	}
	/** Compute support of an array of columns (items). */
	public int supp(int []items) {
		int n = items.length;
		if(n==0)
			return atts[0].size();
		if(n==1)
			return atts[items[0]].bitCount();
		if(n==2)
			return atts[items[0]].and_size(atts[items[1]]);
		Bitmap b = atts[items[0]].clone();
		int n1 = n-1;
		for(int i=1; i<n1; ++i)
			b.and_inplace(atts[items[i]]);
		return b.and_size(atts[items[n1]]);
	}
	/** Compute support of an array of columns (items). */
	public int supp(ArrayList<Integer> items) {
		int n = items.size();
		if(n==0)
			return atts[0].size();
		if(n==1)
			return atts[items.get(0)].bitCount();
		if(n==2)
			return atts[items.get(0)].and_size(atts[items.get(1)]);
		Bitmap b = atts[items.get(0)].clone();
		int n1 = n-1;
		for(int i=1; i<n1; ++i)
			b.and_inplace(atts[items.get(i)]);
		return b.and_size(atts[items.get(n1)]);
	}
}
