/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package rst.table;

import java.util.ArrayList;
import java.util.TreeMap;

/** Column of indexes to elements of a given type. */
public class TypedColumn<T> extends Column {
	/** Value of an index. */
	public ArrayList<T> decode;
	/** Index of a value. */
	public TreeMap<T, Integer> encode;
	
	/** Constructor. */
	public TypedColumn(String name, int size)  {
		super(name, size);		
		decode = new ArrayList<T>();
		encode = new TreeMap<T, Integer>();
	}
	
	/** Number of distinct values. */
	public int getNDistinct() {
		return encode.size();
	}

	/** Set row to a given value. */
	public void set(int pos, T value) {
		Integer idx = encode.get(value);
		if( idx == null ) 
			idx = addValue(value);
		index[pos] = idx; 
	}
	
	/** Add a possible value. Use this method to fix an order between values. */
	public Integer addValue(T value) {
		Integer idx = decode.size();
		encode.put(value, idx);
		decode.add(value);
		return idx;
	}
	
	/** Get value at a given row. */
	public T get(int pos) {
		return decode.get(index[pos]);
	}
	
	public String getString(int pos) {
		return get(pos).toString();
	}
	
	public String getIdxValues(int idx) {
		return decode.get(idx).toString();
	}

	public int getLastIndex() {
		return decode.size()-1;
	}
}